#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// hard coded 95% interval
static const double z = 1.96;
static const double desired_conf = 95.0;

static const int METAREPS = 5;
static const int REPS = 10 * 1000 * 1000;

// return a random value in [0..1]
static double randf (void)
{
  return (double)rand() / RAND_MAX;
}

struct interval {
  double low, high;
};

static struct interval normal_approx_interval (const int n, 
					       const double p)
{
  double plus_minus = z * sqrt (p * (1-p) / n);
  struct interval conf = { p - plus_minus, p + plus_minus };
  return conf;
}

static struct interval wilson_score_interval (const int n, 
					      const double p)
{
  double term1 = p + (z * z) / (2*n);
  double plus_minus = z * sqrt ((p * (1 - p) / n) + (z * z / (4 * n * n)));
  double term2 = 1 + (z * z  / n);
  struct interval conf = { (term1 - plus_minus) / term2, (term1 + plus_minus) / term2 };
  return conf;
}

static void check_confidence (const int n, 
			      const double p)
{
  int inside = 0;
  int i;
  for (i=0; i<REPS; i++) {
    int success = 0;
    int j;
    for (j=0; j<n; j++) {
      if (randf() < p) {
	success++;
      }
    }
    double freq = (double)success / n;
    struct interval conf = normal_approx_interval (n, freq);
    // struct interval conf = wilson_score_interval (n, freq);
    if (p > conf.low &&
	p < conf.high) {
      inside++;
    }
  }
  double pct = 100 * ((double)inside / REPS);
  printf ("measured coverage= %.2lf %%\n", pct);
}

static double dmin (const double a, 
		    const double b)
{
  if (a<b) return a;
  return b;
}

static void usage (void)
{
  printf ("usage: conf n p\n");
  exit (-1);
}

int main (int argc, char *argv[])
{
  srand (time (NULL));

  if (argc != 3) usage();

  int res;
  int n;
  res = sscanf (argv[1], "%d", &n);
  if (res != 1) usage();

  double p;
  res = sscanf (argv[2], "%lf", &p);
  if (res != 1) usage();

  if (n<1 || p<0 || p>1) usage();

  printf ("n= %d, p= %.6lf, nominal coverage= %.2lf %%\n", n, p, desired_conf); 

  if (dmin(n*p,n*(1-p)) < 5) {
    printf ("warning: min(np,n(1-p)) < 5\n");
  } else if (dmin(n*p,n*(1-p)) < 10) {
    printf ("warning: min(np,n(1-p)) < 10\n");
  }

  int i;
  for (i=0; i<METAREPS; i++) {
    check_confidence (n, p);
  }

  return 0;
}
